/* ---------------------------------------------------------------------------
Programme : OPERMATRICE.C
Langage   : C sur UNIX
Date      : Janvier 1993

Auteur    : Jean-Francois Guilbert   GUIJ05016601
Modifie par: F. Meunier.

   Ce module comprend les operations relatives a la gestion des matrices
representant une image.
--------------------------------------------------------------------------- */

#include "image.h"

/* ---------------------------------------------------------------------------
Fonction CREERMATRICE : Cette fonction alloue l'espace necessaire a la matrice
                        de traitement d'image et copie le contenu de imgSV
(image source) dans la matrice.  L'espace pour la matrice destination (imgDM)
est aussi alloue.

appel : image->imgSM = creerMatrice (image)
--------------------------------------------------------------------------- */
Uchar ** creerMatrice (Image *image)
{
  Uchar **matrice;                      /* Pointeur de la matrice retournee */

  int rangee, colonne,                           /* Indices dans la matrice */
      ind = 0;                       /* Indice dans le vecteur source imgSV */

                                                    /* Image matrice source */
  if ((matrice = (Uchar **) calloc (image->tete->rasHauteur, sizeof (Uchar *)))
              == (Uchar **) NULL)
  {
     printf ("\nInfo : Erreur avec 'CALLOC' (crMat1)");
     exit(1);

  } /* if then */

  for (rangee = 0; rangee < image->tete->rasHauteur; rangee++) 
  {
      if ((matrice[rangee] = (Uchar *) calloc (image->tete->rasLargeur,
                                               sizeof(Uchar)))
                          == (Uchar *) NULL)
      {
         printf ("\nInfo : Erreur avec 'CALLOC' (crMat[]1)");
         exit(1);

      } /* if then */

  } /* for */

  for (rangee = 0; rangee <= image->tete->rasHauteur - 1; rangee++)     
      for (colonne = 0; colonne <= image->tete->rasLargeur - 1; colonne++)
          matrice[rangee][colonne] = image->imgSV[ind++];

                                              /* Image matrice destination */
  if ((image->imgDM = (Uchar **)
                      calloc (image->tete->rasHauteur, sizeof (Uchar *)))
                   == (Uchar **) NULL)
  {
     printf ("\nInfo : Erreur avec 'CALLOC' (crMat2)");
     exit(1);

  } /* if then */

  for (rangee = 0; rangee < image->tete->rasHauteur; rangee++) 
  {
      if ((image->imgDM[rangee] = (Uchar *) calloc (image->tete->rasLargeur,
                                                    sizeof(Uchar)))
                               == (Uchar *) NULL)
      {
         printf ("\nInfo : Erreur avec 'CALLOC' (crMat[]2)");
         exit(1);

      } /* if then */

  } /* for */

  return matrice;

} /* creerMatrice */

/* ---------------------------------------------------------------------------
Fonction DETRUIREMATRICE : Cette fonction libere l'espace alloue aux matrices
                           de traitement d'image source et destination.

appel : detruireMatrice (image)
--------------------------------------------------------------------------- */
void detruireMatrice (Image *image)
{
  int rangee;                                     /* Indice dans la matrice */

  if (image->imgSM != (Uchar **) NULL)
  {
     for (rangee = 0; rangee < image->tete->rasHauteur; rangee++) /* Source */
         free (image->imgSM[rangee]);

     free (image->imgSM);

     image->imgSM = (Uchar **) NULL;

     for (rangee = 0; rangee < image->tete->rasHauteur; rangee++) /* Dest. */
         free (image->imgDM[rangee]);

     free (image->imgDM);

     image->imgDM = (Uchar **) NULL;

  } /* if then */

} /* detruireMatrice */

/* ---------------------------------------------------------------------------
Fonction CREERIMAGEDEST : Cette fonction utilise CREERVECTDEST pour l'image
                          destination sous forme vectorielle (imgDV) et copie
le contenu de la matrice destination (imgDM) a ce vecteur.

appel : image->imgDV = creerImageDest (image)
--------------------------------------------------------------------------- */
Uchar * creerImageDest (Image *image)
{
  Uchar *vecteurDest;                   /* Vecteur retourne par la fonction */

  int rangee, colonne,                           /* Indices dans la matrice */
      ind = 0;                       /* Indice dans le vecteur source imgSV */
  
  vecteurDest = creerVectDest (image->tete->rasLongueur);

  for (rangee = 0; rangee <= image->tete->rasHauteur - 1; rangee++)     
      for (colonne = 0; colonne <= image->tete->rasLargeur - 1; colonne++)
          vecteurDest[ind++] = image->imgDM[rangee][colonne];

  return vecteurDest;

} /* creerImageDest */

/* ---------------------------------------------------------------------------
Fonction CREERVECTDEST : Cette fonction alloue l'espace necessaire pour
                         l'image destination sous forme vectorielle (imgDV).

appel : image->imgDV = creerVectDest (image->tete->rasLongeur)
--------------------------------------------------------------------------- */
Uchar * creerVectDest (int longueur)
{
  Uchar *vecteurDest;                   /* Vecteur retourne par la fonction */

  if ((vecteurDest =  (Uchar *) malloc (longueur)) == (Uchar *) NULL)
  {
     printf ("\nInfo : Erreur avec le 'MALLOC' du vecteur destination.\n");
     exit (-5);

  } /* if then */

  return vecteurDest;

} /* creerVectDest */
